﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace WPFDisciples.Common
{
    /// <summary>
    /// Mediator Pattern for better MVVM
    /// visit: http://sachabarber.net/?p=477
    /// and http://marlongrech.wordpress.com/2008/03/20/more-than-just-mvc-for-wpf/
    /// </summary>
    public sealed class Mediator
    {
        #region Data
        static readonly Mediator instance = new Mediator();
        #endregion

        #region Ctor
        /// <summary>
        /// Private instance so that no one is allowed to create an instance of the Mediator.
        /// </summary>
        private Mediator() { }
        #endregion

        #region Public Properties


        public static Mediator Instance
        {
            get
            {
                return instance;
            }
        }

        #endregion

        #region Public Methods

        MultiDictionary<string, WeakDelegate> invocationList
            = new MultiDictionary<string, WeakDelegate>();
        public void Register(string message, object target, string method)
        {
            invocationList.AddValue(message, new WeakDelegate(target, method));
        }

        /// <summary>
        /// Notify all colleagues that are registed to the 
        /// specific message
        /// </summary>
        /// <param name="message">The message for the notify by</param>
        /// <param name="args">The arguments for the message</param>
        public void NotifyColleagues(string message,
            object args)
        {
            if (invocationList.ContainsKey(message))
            {
                for (int i = 0; i < invocationList[message].Count; i++)
                {
                    var weakDelegate = invocationList[message][i];
                    if (weakDelegate.IsAlive)
                        weakDelegate.Notify(args);
                    else//remove the weak delegate from the invokation list
                    {
                        invocationList[message].RemoveAt(i);
                        i--;
                    }
                }
            }
        }
        #endregion

        class WeakDelegate : WeakReference
        {
            public string Method { get; private set; }

            private MethodInfo methodInfo;

            public WeakDelegate(object target, string method)
                : base(target)
            {
                Method = method;
                methodInfo = Target.GetType().GetMethod(Method, BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static);
                if (methodInfo == null)
                    throw new InvalidOperationException(String.Format("Method {0} was not found on target object", Method));
            }

            public void Notify(object args)
            {
                if (IsAlive)
                    methodInfo.Invoke(Target, new[] { args });
            }
        }
    }
}
